package com.kdanmobile.admanager.admob

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import com.google.ads.mediation.facebook.FacebookAdapter
import com.google.android.gms.ads.AdListener
import com.google.android.gms.ads.AdLoader
import com.google.android.gms.ads.formats.MediaView
import com.google.android.gms.ads.formats.NativeAdOptions
import com.google.android.gms.ads.formats.UnifiedNativeAd
import com.google.android.gms.ads.formats.UnifiedNativeAdView
import com.kdanmobile.admanager.NativeAdListener
import com.kdanmobile.admanager.NativeAdManager
import com.kdanmobile.admanager.NativeAdViewBinder
import com.kdanmobile.admanager.R
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.CopyOnWriteArrayList

class AdmobNativeAdManager(private val context: Context) : NativeAdManager {

    override var isPersonalized = true
    override val listener = CopyOnWriteArrayList<NativeAdListener>()

    private val nativeAdMap = ConcurrentHashMap<String, UnifiedNativeAd>()
    private val isLoadingMap = ConcurrentHashMap<String, Boolean>()

    override fun request(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder) {
        if (isLoading(adUnitId)) return

        isLoadingMap[adUnitId] = true
        AdLoader.Builder(context, adUnitId)
                .forUnifiedNativeAd {
                    nativeAdMap[adUnitId] = it
                    isLoadingMap[adUnitId] = false
                    onLoaded(adUnitId)
                }.withAdListener(object : AdListener() {
                    override fun onAdFailedToLoad(p0: Int) {
                        super.onAdFailedToLoad(p0)
                        isLoadingMap[adUnitId] = false
                        onFailedToLoad(adUnitId)
                    }

                    override fun onAdImpression() {
                        super.onAdImpression()
                        onImpressed(adUnitId)
                    }

                    override fun onAdClicked() {
                        super.onAdClicked()
                        onClicked(adUnitId)
                    }
                }).withNativeAdOptions(NativeAdOptions.Builder().build())
                .build()
                .loadAd(RequestFactory.create(isPersonalized))
    }

    override fun takeAdView(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder):View? {
        val ad = nativeAdMap.remove(adUnitId) ?: return null
        return buildAdView(ad, nativeAdViewBinder)
    }

    override fun isLoading(adUnitId: String): Boolean {
        return isLoadingMap[adUnitId] ?: false
    }

    private fun onLoaded(adUnitId: String) {
        listener.forEach {
            it.onLoaded(adUnitId)
        }
    }

    private fun onImpressed(adUnitId: String) {
        listener.forEach { it.onImpressed(adUnitId) }
    }

    private fun onClicked(adUnitId: String) {
        listener.forEach { it.onClicked(adUnitId) }
    }

    private fun onFailedToLoad(adUnitId: String) {
        listener.forEach {
            it.onFailedToLoad(adUnitId)
        }
    }

    private fun buildAdView(ad: UnifiedNativeAd, nativeAdViewBinder: NativeAdViewBinder): View {
        return (LayoutInflater.from(context).inflate(nativeAdViewBinder.layout, null) as UnifiedNativeAdView).apply {
            /**
             * Only the ad titles, URLs, CTA, and non-background image assets should be clickable.
             * https://support.google.com/admob/answer/6240814?hl=en&ref_topic=2745287
             * */
            findViewById<MediaView>(nativeAdViewBinder.mainImageViewId)?.let {
                mediaView = it
            }
            findViewById<ImageView>(nativeAdViewBinder.iconImageViewId)?.let {
                ad.icon?.apply { it.setImageDrawable(this.drawable) }
                iconView = it
            }
            findViewById<TextView>(nativeAdViewBinder.titleTextViewId)?.let {
                ad.headline?.apply { it.text = this }
                headlineView = it
            }
            findViewById<TextView>(nativeAdViewBinder.textTextViewId)?.let {
                ad.body?.apply { it.text = this }
            }
            findViewById<Button>(nativeAdViewBinder.ctaButtonId)?.let {
                ad.callToAction?.apply { it.text = this }
                callToActionView = it
            }
            findViewById<TextView>(nativeAdViewBinder.sponsoredTextView)?.let {
                it.setText(when {
                    ad.extras.containsKey(FacebookAdapter.KEY_SOCIAL_CONTEXT_ASSET) -> {
                        R.string.advertisement_sponsored_text_fb
                    }
                    else -> R.string.advertisement_sponsored_text_admob
                })
            }
            post {
                setNativeAd(ad)
            }
        }
    }
}